<?php

/**
 * @file
 */
 
class views_node_taxonomy_filter_handler_filter_tid extends views_handler_filter {

  /**
   * Determine if a filter can be exposed.
   */
  function can_expose() { return FALSE; }

  function option_definition() {
    $options = parent::option_definition();
    $options['operator'] = array('default' => 'is');

    return $options;
  }
  
  function operator_options() {
    if ($this->options['multiple']) {
      return array(
        'is' => t('Is one of'),
        'is not' => t('Is not one of'),
      );
    }
    else {
      return array(
        'is' => t('Is'),
        'is not' => t('Is not'),
      );
    }
  }
    
  /**
   * Supply an edit form for the Filter.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $vocabs = array();
    $result = db_query("SELECT v.vid, v.name FROM {taxonomy_vocabulary} v ");
    //while($vocab = db_fetch_object($result)) {
    foreach ($result as $vocab) {
      $vocabs[$vocab->vid] = $vocab->name;
    }
      
    $default_value = $this->options['value']; 
    
    $form['value'] = array(
      '#type' => 'select',
      '#title' => t('Vocabulary'),
      '#options' => $vocabs,
      '#default_value' => $default_value,
      '#description' => t('Select the Vocabulary.'),
      '#multiple' => TRUE,
    );
  }
  
  /**
   * Display the value of the Filter selection in the Views edit screens.
   */
  function admin_summary() {
    $vocabularies =  taxonomy_get_vocabularies();
     
    if (is_array($this->value)) {
      $vocab_name = '';
      foreach ($this->value as $v) {
        $vocab = $vocabularies[$v];        
        $vocab_name .= $vocab->name . ', ';
      }
    }
    else {
      $vocab = $vocabularies[$this->value];
      $vocab_name = $vocab->name;
    }
    return t("for Vocabulary: @vocab_name", array('@vocab_name' => $vocab_name));
  }

  /**
   * Modify the query appropriately.
   */
  function query() {
    if (empty($this->value)) {
      return;
    }
    $allowed_vocabs = $this->value;
    
    if (arg(0) != 'node') {
      return;
    }
    
    $nid = arg(1);
    if (!is_numeric($nid)) {
      return;
    }
    $node = node_load($nid);
    
    $fields = field_info_instances('node', $node->type);
    
    $tids = array();
    
    foreach ($fields as $field_name => $field) {
      // check if this is a "taxonomy field"
      if (isset($field['display']['default']['module']) && $field['display']['default']['module'] == 'taxonomy') {
        if (isset($node->{$field_name}['und']) && !empty($node->{$field_name}['und'])) {
          foreach ($node->{$field_name}['und'] as $term) {
            // keep terms in authorized vocabularies
            if (in_array($term['taxonomy_term']->vid, $allowed_vocabs)) {
              $tids[] = $term['tid'];
            }
          }
        }
      }
    }
      
    if (sizeof($tids) < 1) {
      return;
    }
    $tids = implode(',', $tids);
    $alias = $this->query->ensure_table('taxonomy_index');
    $this->query->add_where_expression($this->options['group'], "$alias.tid IN ('$tids')");
  }
}